using Autofac;
using AutoMapper;
using Xunit;
using ZooBL.Configs;
using ZooBL.Dto.Animal;
using ZooBL.Services;
using ZooDAL.Data;
using ZooDAL.Models;

namespace ZooTests;

public class AnimalServiceTests
{
    private static readonly IAnimalService AnimalService = AutofacBLConfig.Configure().Resolve<IAnimalService>();
    private static readonly IMapper Mapper = new Mapper(new MapperConfiguration(BusinessMappingConfig.ConfigureMapping));

    [Fact]
    public async Task AnimalExists_GetAnimalByIdAsync_Animal()
    {
        // Arrange


        // Act
        var result = await AnimalService.GetAnimalByIdAsync(DataInitializer.SuricataTomId);

        //Assure
        Assert.NotNull(result);
    }

    [Fact]
    public async Task AnimalExists_GetAnimalDetailByIdAsync_Animal()
    {
        // Arrange


        // Act
        var result = await AnimalService.GetAnimalDetailByIdAsync(DataInitializer.SuricataTomId);

        //Assure
        Assert.NotNull(result);
        Assert.NotNull(result.Enclosure);
    }

    [Fact]
    public static async Task? ExistingAnimal_DeleteAnimalAsync_AnimalDeleted()
    {
        // Arrange
        var helper = new Helper();
        var animal = new Animal
        {
            Id = Guid.NewGuid(),
            Name = "TEST",
            Specie = "TEST"
        };
        await helper.InsertIntoDbAsync(animal);

        // Act
        await AnimalService.DeleteAnimalAsync(animal.Id);

        //Assure
        var actual = await helper.GetEntityByIdAsync<Animal>(animal.Id);
        Assert.Null(actual);
    }

    [Fact]
    public static async Task? Animal_UpdateAnimalAsync_AnimalChanged()
    {
        // Arrange
        var helper = new Helper();
        var animal = new Animal
        {
            Id = Guid.NewGuid(),
            Name = "TEST",
            Specie = "TEST"
        };
        var animalDto = Mapper.Map<AnimalDto>(animal);
        animalDto.Name = "CHANGED";
        await helper.InsertIntoDbAsync(animal);

        // Act
        await AnimalService.UpdateAnimalAsync(animal.Id, animalDto);

        //Assure
        var actual = await helper.GetEntityByIdAsync<Animal>(animalDto.Id);
        Assert.NotNull(actual);
        Assert.Equal(animalDto.Name, actual.Name);
        Assert.Equal(animalDto.Specie, actual.Specie);
    }

    [Fact]
    public static async Task? Animal_CreateAnimal_AnimalCreated()
    {
        // Arrange
        var helper = new Helper();
        var animalDto = new AnimalDto
        {
            Id = Guid.NewGuid(),
            Name = "TEST",
            Specie = "TEST"
        };

        // Act
        await AnimalService.CreateAnimalAsync(animalDto);

        //Assure
        var actual = await helper.GetEntityByIdAsync<Animal>(animalDto.Id);
        Assert.NotNull(actual);
    }


    [Fact]
    public static async Task AnimalsExists_GetAllAnimalsAsync_NonEmptyResult()
    {
        // Arrange


        // Act
        var result = await AnimalService.GetAllAnimalsAsync();

        //Assure
        Assert.NotEmpty(result);
    }

    [Fact]
    public static async Task? NonExistingAnimal_GetAnimalByIdAsync_Null()
    {
        // Arrange

        // Act
        var result = await AnimalService.GetAnimalByIdAsync(Guid.NewGuid());

        //Assure
        Assert.Null(result);
    }
}