using Autofac;
using AutoMapper;
using Xunit;
using ZooBL.Configs;
using ZooBL.Dto.Enclosure;
using ZooBL.Services;
using ZooDAL.Data;
using ZooDAL.Models;

namespace ZooTests;

public class EnclosureServiceTests
{
    private static readonly IEnclosureService EnclosureService = AutofacBLConfig.Configure().Resolve<IEnclosureService>();
    private static readonly IMapper Mapper = new Mapper(new MapperConfiguration(BusinessMappingConfig.ConfigureMapping));

    [Fact]
    public async Task EnclosureExists_GetEnclosureByIdAsync_Enclosure()
    {
        // Arrange


        // Act
        var result = await EnclosureService.GetEnclosureByIdAsync(DataInitializer.EnclosureSuricataId);

        //Assure
        Assert.NotNull(result);
    }

    [Fact]
    public async Task EnclosureExists_GetEnclosureDetailByIdAsync_Enclosure()
    {
        // Arrange


        // Act
        var result = await EnclosureService.GetEnclosureDetailByIdAsync(DataInitializer.EnclosureSuricataId);

        //Assure
        Assert.NotNull(result);
        Assert.NotNull(result.Animals);
    }

    [Fact]
    public static async Task? ExistingEnclosure_DeleteEnclosureAsync_EnclosureDeleted()
    {
        // Arrange
        var helper = new Helper();
        var enclosure = new Enclosure
        {
            Id = Guid.NewGuid(),
            Length = 500,
            Width = 750
        };
        await helper.InsertIntoDbAsync(enclosure);

        // Act
        await EnclosureService.DeleteEnclosureAsync(enclosure.Id);

        //Assure
        var actual = await helper.GetEntityByIdAsync<Enclosure>(enclosure.Id);
        Assert.Null(actual);
    }

    [Fact]
    public static async Task? Enclosure_UpdateEnclosureAsync_EnclosureChanged()
    {
        // Arrange
        var helper = new Helper();
        var enclosure = new Enclosure
        {
            Id = Guid.NewGuid(),
            Length = 500,
            Width = 750
        };
        var enclosureDto = Mapper.Map<EnclosureDto>(enclosure);
        enclosureDto.Length = 1000;
        await helper.InsertIntoDbAsync(enclosure);

        // Act
        await EnclosureService.UpdateEnclosureAsync(enclosureDto.Id, enclosureDto);

        //Assure
        var actual = await helper.GetEntityByIdAsync<Enclosure>(enclosureDto.Id);
        Assert.NotNull(actual);
        Assert.Equal(enclosureDto.Length, actual.Length);
        Assert.Equal(enclosureDto.Width, actual.Width);
    }

    [Fact]
    public static async Task? Enclosure_CreateEnclosure_EnclosureCreated()
    {
        // Arrange
        var helper = new Helper();
        var enclosureDto = new EnclosureDto
        {
            Id = Guid.NewGuid(),
            Length = 500,
            Width = 750
        };

        // Act
        await EnclosureService.CreateEnclosureAsync(enclosureDto);

        //Assure
        var actual = await helper.GetEntityByIdAsync<Enclosure>(enclosureDto.Id);
        Assert.NotNull(actual);
    }


    [Fact]
    public static async Task EnclosuresExists_GetAllEnclosuresAsync_NonEmptyResult()
    {
        // Arrange


        // Act
        var result = await EnclosureService.GetAllEnclosuresAsync();

        //Assure
        Assert.NotEmpty(result);
    }

    [Fact]
    public static async Task? NonExistingEnclosure_GetEnclosureByIdAsync_Null()
    {
        // Arrange

        // Act
        var result = await EnclosureService.GetEnclosureByIdAsync(Guid.NewGuid());

        //Assure
        Assert.Null(result);
    }

    [Fact]
    public static async Task? Enclosure_AddAnimalToEnclosureAsync_AnimalAdded()
    {
        // Arrange
        var helper = new Helper();
        var enclosure = new Enclosure
        {
            Id = Guid.NewGuid(),
            Length = 500,
            Width = 750
        };
        var enclosureDto = Mapper.Map<EnclosureDto>(enclosure);
        await helper.InsertIntoDbAsync(enclosure);

        // Act
        await EnclosureService.AddAnimalToEnclosureAsync(enclosureDto.Id, DataInitializer.TigerJackId);

        //Assure
        var actual = await helper.GetEntityByIdWithIncludesAsync<Enclosure>(enclosureDto.Id, nameof(Enclosure.Animals));
        Assert.NotNull(actual);
        Assert.NotNull(actual.Animals);
        Assert.NotEmpty(actual.Animals);
        Assert.Equal(actual.Animals.First().Id, DataInitializer.TigerJackId);
    }
}